import { __spreadArray } from "tslib";
import * as React from 'react';
import { Platform } from 'react-native';
import { memoize } from '@fluentui-react-native/memo-cache';
var shouldAllowShiftCtrlKeys = Platform.OS === 'win32';
/**
 * Verifies if nativeEvent contains modifier key. The modifier keys that should
 * be taken into account differ based on platform
 * @param nativeEvent
 * @returns `true` if one or more of modifier keys are `true`
 */
var isModifierKey = function (nativeEvent) {
    if (shouldAllowShiftCtrlKeys) {
        return nativeEvent && (nativeEvent.alt || nativeEvent.altKey || nativeEvent.meta || nativeEvent.metaKey);
    }
    else {
        return (nativeEvent &&
            (nativeEvent.alt ||
                nativeEvent.altKey ||
                nativeEvent.ctrl ||
                nativeEvent.ctrlKey ||
                nativeEvent.meta ||
                nativeEvent.metaKey ||
                nativeEvent.shift ||
                nativeEvent.shiftKey));
    }
};
function keyPressCallback(userCallback) {
    var keys = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        keys[_i - 1] = arguments[_i];
    }
    var onKeyEvent = function (e) {
        if (userCallback !== undefined && !isModifierKey(e.nativeEvent) && (keys === undefined || keys.includes(e.nativeEvent.key))) {
            userCallback(e);
            e.stopPropagation();
        }
    };
    return onKeyEvent;
}
function getKeyUpPropsWorker(userCallback) {
    var keys = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        keys[_i - 1] = arguments[_i];
    }
    var keyboardProps = Platform.select({
        ios: undefined,
        android: undefined,
        macos: {
            onKeyUp: keyPressCallback.apply(void 0, __spreadArray([userCallback], keys, false)),
            validKeysUp: keys,
        },
        windows: {
            /**
             * https://github.com/microsoft/react-native-windows/issues/11049
             * Windows doesn't filter on `key` but on `code`, which is quite different ('A' vs 'KeyA' or 'GamepadA').
             * While this discrepancy is present, let's not specify `keyUpEvents`.
             */
            onKeyUp: keyPressCallback.apply(void 0, __spreadArray([userCallback], keys, false)),
        },
        // win32
        default: {
            onKeyUp: keyPressCallback.apply(void 0, __spreadArray([userCallback], keys, false)),
            keyUpEvents: keys.map(function (keyCode) {
                return { key: keyCode };
            }),
        },
    });
    return keyboardProps;
}
function getKeyDownPropsWorker(userCallback) {
    var keys = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        keys[_i - 1] = arguments[_i];
    }
    var keyboardProps = Platform.select({
        ios: undefined,
        android: undefined,
        macos: {
            onKeyDown: keyPressCallback.apply(void 0, __spreadArray([userCallback], keys, false)),
            validKeysDown: keys,
        },
        windows: {
            /**
             * https://github.com/microsoft/react-native-windows/issues/11049
             * Windows doesn't filter on `key` but on `code`, which is quite different ('A' vs 'KeyA' or 'GamepadA').
             * While this discrepancy is present, let's not specify `keyDownEvents`.
             */
            onKeyDown: keyPressCallback.apply(void 0, __spreadArray([userCallback], keys, false)),
        },
        // win32
        default: {
            onKeyDown: keyPressCallback.apply(void 0, __spreadArray([userCallback], keys, false)),
            keyDownEvents: keys.map(function (keyCode) {
                return { key: keyCode };
            }),
        },
    });
    return keyboardProps;
}
/**
 * Re-usable hook for an onKeyUp event.
 * @param userCallback The function you want to be called once the key has been activated on key up
 * @param keys A string of the key you want to perform some action on. If undefined, always invokes userCallback
 * @returns KeyPressProps: An object containing the correct platform specific props to handle key press
 */
export var useKeyUpProps = memoize(getKeyUpPropsWorker);
/**
 * Re-usable hook for an onKeyDown event.
 * @param userCallback The function you want to be called once the key has been activated on key down
 * @param keys A string of the key you want to perform some action on. If undefined, always invokes userCallback
 * @returns KeyPressProps: An object containing the correct platform specific props to  handle key press
 */
export var useKeyDownProps = memoize(getKeyDownPropsWorker);
/** Exposes the behavior of useKeyProps for the current platform as a boolean */
export var preferKeyDownForKeyEvents = Platform.select({
    macos: true,
    default: false,
});
/**
 * Re-usable hook for keyboard events. on macOS, this is onKeyDown, while on windows this is onKeyUp.
 * @param userCallback The function you want to be called once the key has been activated on key down
 * @param keys A string of the key you want to perform some action on. If undefined, always invokes userCallback
 * @returns KeyPressProps: An object containing the correct platform specific props to handle key press
 */
export var useKeyProps = preferKeyDownForKeyEvents ? useKeyDownProps : useKeyUpProps;
/**
 * Re-usable hook for an onKeyDown event.
 * @param userCallback The function you want to be called once the key has been activated on key up
 * @param keys A string of the key you want to perform some action on. If undefined, always invokes userCallback
 * @returns onKeyEvent() - Callback to determine if key was pressed, if so, call userCallback
 * @deprecated use the hook `useKeyProps` instead
 */
export function useKeyCallback(userCallback) {
    var keys = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        keys[_i - 1] = arguments[_i];
    }
    var onKeyEvent = React.useCallback(function (e) {
        if (userCallback !== undefined && (keys === undefined || keys.includes(e.nativeEvent.key))) {
            userCallback(e);
            e.stopPropagation();
        }
    }, [keys, userCallback]);
    return onKeyEvent;
}
//# sourceMappingURL=useKeyProps.js.map